#ifndef DEVICEELEMENTWIDGET_H
#define DEVICEELEMENTWIDGET_H

#include "gmc-miscellaneousse_global.h"

#include <elementwidget.h>

#include "ruledeviceserviceexpressions.h"

class QValidator;
class QSortFilterProxyModel;

class TreeNode;
class TreeNodeModel;
class TreeNodeSortFilterProxyModel;
class ChoiceTreeNodeProxyModel;
class OneCheckTreeNodeModel;
class AuditComponentRootTreeNode;
class AuditTwoCheckTreeNodeModel;

namespace Ui
{
class DeviceElementWidget;
}

class GMCMISCELLANEOUSSESHARED_EXPORT DeviceElementWidget
: public ViewEditElementWidget
, public RuleDeviceExpressionsServiceClass
{

    Q_OBJECT

public:

    DeviceElementWidget( Module & /*module*/
                       , bool ( * /*isEditable*/ )( const Module & ), QWidget * /*parent*/ = 0 );
    virtual ~DeviceElementWidget();

    void init();

    bool nameEditableEditMode() const;
    void setNameEditableEditMode( bool nameEditableEditMode );

    virtual void beforeCreation();
    virtual ElementData * exportElementData() const;
    virtual void afterSuccessfulRead( Element * /*testElement*/, ElementData * /*elementData*/ );
    virtual void afterSuccessfulUpdate();

    virtual QString caption() const;

protected:

    virtual void reset();
    virtual bool isContentValid() const;
    virtual void blockEditWidget( bool /*blocked*/ );

    QString auditToString( qulonglong /*audit*/ ) const;

protected slots:

    void auditChanged();
    void importRecordData();
    void descriptionChanged();

protected:

    Ui::DeviceElementWidget *_ui;
    bool _nameEditableEditMode;
    int _descriptionMaxSize;
    QValidator *_nameValidator;

    TreeNode *_usersRootNode;
    TreeNodeModel *_usersTreeNodeModel;
    QSortFilterProxyModel *_usersProxyModel;

    TreeNode *_groupsRootNode;
    TreeNodeModel *_groupsTreeNodeModel;
    QSortFilterProxyModel *_groupsProxyModel;

    TreeNode *_categoriesRootNode;
    OneCheckTreeNodeModel *_categoriesTreeNodeModel;
    TreeNodeSortFilterProxyModel *_categoriesProxyModel;

    TreeNode *_levelsRootNode;
    ElementModel *_levelsElementModel;
    QSortFilterProxyModel *_levelsProxyModel;

    AuditTwoCheckTreeNodeModel *_auditTreeNodeModel;
    QSortFilterProxyModel *_auditProxyModel;

    TreeNode *_rulesRootNode;
    TreeNodeModel *_rulesTreeNodeModel;
    ChoiceTreeNodeProxyModel *_rulesChoiceModel;

};

#endif // DeviceElementWidget_H
